/**
 * synopsis: Generates an JWM menu xml segment based on an XDG menu structure
 * purpose: The purpose of this utility is to keep the JWM menu 
 *          synchronized with the XDG configuration
 * usage: jwm-xdgmenu <xdgMenuFile>
 * author: Raul Suarez (rarsa at yahoo dot com)
 * Licence: GPL
 *
 * Dependencies:
 *   gnome-menus 2.12
 *   glib
 */

#include <glib.h>
#include <glib/gprintf.h>
#include <gmenu-tree.h>
#include <string.h>

/*=============================================================================
 * Declarations 
 */
static void show_help();
static void process_directory(GMenuTreeDirectory *directory);
static void process_entry(GMenuTreeEntry *entry);
static int menu_height = 0;

/*=============================================================================
 * Main Function 
 */
int
main (int argc, char **argv)
{
  char *xdgfile = NULL;
  for (int i = 1; i < argc; i++) {
	  if ( strcmp(argv[i],"-h")==0 ||
	       strcmp(argv[i],"-help")==0 ||
	       strcmp(argv[i],"--help")==0) {
		show_help();
		return 1;
		
	  } else if ( strcmp(argv[i],"-m") == 0) {
		i++;
		if (i<argc) {
			menu_height = atoi(argv[i]);
		}

      } else {
		  xdgfile = argv[i];
	  }
  }
  if (!xdgfile) return 1;

  GMenuTree *menuTree = gmenu_tree_lookup (xdgfile,  GMENU_TREE_FLAGS_NONE);

  GMenuTreeDirectory *rootDirectory = gmenu_tree_get_root_directory(menuTree);

  process_directory(rootDirectory);
  
  gmenu_tree_item_unref (rootDirectory);

  return 0;

}

/*=============================================================================
 * Shows usage parameters
 */
void
show_help() 
{
  printf ("Creates a JWM inlcude menu from an XDG menu structure.\n");
  printf ("Usage:\n");
  printf ("  jwm-xdgmenu [-m menu-height] xdgmenufile\n");
  printf ("\n");
  printf ("  xdgmenufile: Fully qualified path to the XDG menu file. This is a mandatory parameter\n");
  printf ("\n");
  printf ("For example:\n");
  printf ("  jwm-xdgmenu \"/etc/xdg/menus/applications.menu\"\n\n");
}

/*=============================================================================
 * This function processes a directory entry and all it's child nodes 
 */
void
process_directory(GMenuTreeDirectory *directory)
{
  if (menu_height) {
    g_printf("<Menu label=\"%s\" icon=\"%s\" height=\"%d\">\n", 
            gmenu_tree_directory_get_name(directory),
            gmenu_tree_directory_get_icon(directory),
            menu_height);
  } else {
    g_printf("<Menu label=\"%s\" icon=\"%s\">\n", 
            gmenu_tree_directory_get_name(directory),
            gmenu_tree_directory_get_icon(directory));
  }

  GMenuTreeItemType entryType;

  GSList *entryList = gmenu_tree_directory_get_contents (directory);

  GSList *l;

  for (l = entryList; l; l = l->next)
  {
 
    GMenuTreeItem *item = l->data;

    entryType = gmenu_tree_item_get_type (GMENU_TREE_ITEM(item));

    if (entryType == GMENU_TREE_ITEM_DIRECTORY)
    {
      process_directory(GMENU_TREE_DIRECTORY(item));
    }
    else if (entryType == GMENU_TREE_ITEM_ENTRY)
      process_entry(GMENU_TREE_ENTRY(item));
    
    gmenu_tree_item_unref (item);

  }
  g_printf("</Menu>\n");
  g_slist_free (entryList);
}

/*=============================================================================
 * This function adds an application entry
 */
void
process_entry(GMenuTreeEntry *entry) 
{

  //JWM doesn't support %F and others, so get rid of it
  const gchar *exec = gmenu_tree_entry_get_exec (entry);
  gchar *ptr = strstr(exec, " %");
  if (ptr != NULL) {
    *ptr = '\0';
  }

  g_printf("<Program label=\"%s\" icon=\"%s\">%s</Program>\n",
            gmenu_tree_entry_get_name(entry),
            gmenu_tree_entry_get_icon (entry),
            exec);
}
